
-- The minimum cooldown duration to make the button fading out
local MIN_DURATION = 1.5

-- Alpha value for completly faded out button
local FADED_ALPHA_VALUE = 0.15

-- Globals made upvalues
local _G = _G
local GetTime = GetTime
local next = next
local pairs = pairs

-- Locals

local function Anim_End(animGroup, ...)
	if animGroup.alpha then
		animGroup:GetParent():SetAlpha(animGroup.alpha)
		animGroup.alpha = nil
	end
	animGroup.cooldown:SetAlpha(1)
end

local function InitFrame(frame, cooldown)
	local animGroup = frame:CreateAnimationGroup()
	animGroup:SetScript('OnStop', Anim_End)
	animGroup:SetScript('OnFinished', Anim_End)
	animGroup.cooldown = cooldown
	frame.CF_animGroup = animGroup
	
	local anim = animGroup:CreateAnimation("Alpha")
	anim:SetOrder(1)
	anim:SetSmoothing("IN")
	frame.CF_anim = anim
end

local frames = setmetatable({}, { __index = function(t, cooldown)
	local frame = false
	if not cooldown.noCooldownCount then
		frame = cooldown:GetParent()
		local name = frame:GetName()
		if name and (strmatch(name, '[bB]uff%d*$') or strmatch(name, '[bB]uffIcon%d*$')) then
			frame = false
		else
			InitFrame(frame, cooldown)	
		end
	end
	t[cooldown] = frame
	return frame
end})

local function updateCooldown(cooldown, start, duration)
	local frame = frames[cooldown]
	if not frame then
		return -- do not mess with auras
	end
		
	local animGroup = frame.CF_animGroup
	if start ~= animGroup.start and duration ~= animGroup.duration then
		animGroup.start, animGroup.duration = start, duration
		
		local alpha = animGroup.alpha or frame:GetAlpha()
		if animGroup:IsPlaying() then
			animGroup:Stop()
		end
		if start > 0 and duration > MIN_DURATION then
			local timeLeft = start + duration - GetTime()
			local anim = frame.CF_anim
			anim:SetDuration(min(timeLeft, 5))
			anim:SetStartDelay(max(0, timeLeft - 5))
			anim:SetChange(alpha * (1 - FADED_ALPHA_VALUE))			
			frame:SetAlpha(alpha * FADED_ALPHA_VALUE)
			animGroup.alpha = alpha
			animGroup:Play()
			cooldown:SetAlpha(0)
		end
	end
end

hooksecurefunc(getmetatable(ActionButton1Cooldown).__index, 'SetCooldown', updateCooldown)


